/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "projectsdialog.h"
#include "ui_projectsdialog.h"

#include <QDebug>
#include <QFileDialog>
#include <QMessageBox>

ProjectsDialog::ProjectsDialog(QWidget *parent)
    : QDialog(parent), data(ProjectList::instance()), ui(new Ui::ProjectsDialog) {
    ui->setupUi(this);
    // populate list
    syncList();
    if (data.size() < 2) {
        on_newButton_clicked();
    }
}

ProjectsDialog::~ProjectsDialog() { delete ui; }

void ProjectsDialog::syncList() {
    ui->listWidget->clear();
    for (int i = 0; i < data.size(); i++) {
        Project &project = data.get(i);
        QString listString;
        for (int j = 0; j < project.folderCount(); j++) {
            if (j > 0) {
                listString.append(", ");
            }
            listString.append(project.folder(j).path);
        }
        ui->listWidget->addItem(listString);
    }
}

void ProjectsDialog::on_newButton_clicked() {
    QString root(QDir::homePath());
    if (data.size() > 0) {
        // calculate common root of existing projects
        root = data.commonRoot();
    }
    QString path = QFileDialog::getExistingDirectory(this, "Open Project Folder", root);
    if (path.length() > 0) {
        // matches existing projects?
        int index = data.findProject(path);
        if (index > -1) {
            ui->listWidget->setCurrentRow(index);
        } else {
            // need new
            QFileInfo fileInfo(path);
            NamedFile folder(fileInfo.fileName(), fileInfo.filePath());
            Project project;
            project.addFolder(folder);
            data.addProject(project);
            syncList();
            ui->listWidget->setCurrentRow(data.size() - 1);
        }
    }
}

void ProjectsDialog::on_openButton_clicked() {
    bool error = false;
    if (!data.isOpen(ui->listWidget->currentRow())) {
        if (data.open(ui->listWidget->currentRow())) {
            emit projectChanged();
        } else {
            error = true;
        }
    } else if (!data.currentExists()) {
        error = true;
    }
    if (error) {
        QString message("Cannot open project: folder does not exist");
        QMessageBox::warning(this, "Missing Project", message);
    } else {
        this->close();
    }
}

void ProjectsDialog::on_removeButton_clicked() {
    QMessageBox msgBox(this);
    msgBox.setText("Remove Project");
    msgBox.setInformativeText("Do you want to remove this project?");
    msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
    msgBox.setDefaultButton(QMessageBox::Cancel);
    int ret = msgBox.exec();
    if (ret == QMessageBox::Ok) {
        int index = ui->listWidget->currentRow();
        data.removeProject(index);
        syncList();
    }
}

void ProjectsDialog::on_listWidget_itemSelectionChanged() {
    int selection = ui->listWidget->currentRow();
    bool selected(selection != -1);
    ui->openButton->setEnabled(selected);
    // disable remove button for open project
    ui->removeButton->setEnabled(selected && !data.isOpen(selection));
}
